/* Adam's IO library. */

#ifndef NULL
#define NULL 0
#endif

/* A variable-length buffer for arbitrary data (i.e. nulls within it
   don't matter unless you're using bstr(). To declare a buffer, say
   something like "buffer b = BUFFER;". */
typedef struct {
	/* The data. */
	char *s;
	/* The length of the data. */
	int len;
	/* The size of the area that's actually allocated. */
	int size;
} buffer;
#define BUFFER {NULL, 0, 0}

/* A generic IO stream. */
struct _io {
	/* The type of the stream (IOTYPE_x macros below). */
	int type;
	union {
		/* IOTYPE_FD: the file descriptor. */
		int fd;
		/* IOTYPE_SOCK; the socket descriptor. */
		int sock;
		/* IOTYPE_WBUF/RBUF */
		struct {
			char *buf;
			int len;
			int start;
			int end;
			struct _io *from;
			int close;
		} b;
	} u;
};
typedef struct _io io;
#define IOTYPE_FD 0
#define IOTYPE_SOCK 1
#define IOTYPE_RBUF 2
#define IOTYPE_WBUF 3

/* istdinit() must be run before using any of these. */
/* Standard input. */
io *iin;
/* Buffered standard input. */
io *ibin;
/* Standard output. */
io *iout;
/* Buffered standard output. */
io *ibout;
/* Standard error. */
io *ierr;
/* Buffered standard error. */
io *iberr;

/* Resize a buffer, keeping the existing contents. */
void bsetsize(buffer *b, int len);
/* Destroy a buffer. */
void bfree(buffer *b);
/* Set a buffer to a C string. */
void bmake(buffer *b, const char *s);
/* Append a block of memory to a buffer. */
void bappendm(buffer *dest, const char *src, int len);
/* Append a C string to a buffer. */
void bappends(buffer *dest, const char *s);
/* Append a byte to a buffer. */
void bappendc(buffer *dest, char c);
/* Append a buffer to another buffer. */
void bappend(buffer *dest, const buffer *src);
/* Get the length of a buffer. */
int blength(const buffer *b);
/* Get a buffer as a C string. */
const char *bstr(const buffer *b);
/* Trim whitespace from the end of a buffer. */
void btrimr(buffer *b);
/* Replace character from with character to throughout buffer. */
void breplacec(buffer *b, char from, char to);

/* Write the entire contents of a buffer to an io.
   Return 0 on success, -1 on error. */
int bwritea(const buffer *b, io *io);
/* Read up to n bytes from an io onto the end of a buffer.
   Return the number of bytes read, or -1 on error, or 0 on EOF. */
int bread(buffer *b, io *io, int n);
/* Read until EOF from an io onto the end of a buffer.
   Return 0 on success, -1 on error. */
int breada(buffer *b, io *io);
/* Read a string terminated by term from io onto the end of buffer. Does not
   write term into the buffer. Return 1 on success, 0 on EOF, -1 on error.
   Since this reads a character at a time, it will be inefficient unless
   io is an rbuf. */
int breadterm(buffer *b, io *io, char term);
/* Read a string terminated by \n onto the end of buffer. Does not write
   \n into the buffer. Return 1 on success, 0 on EOF, -1 on error. */
int breadline(buffer *b, io *io);

/* Open the standard ios. */
void istdinit(void);
/* Open a file as an io, read-only.  Returns NULL on error. */
io *iopenr(const char *name);
/* Open a file as a buffered io, read-only. Returns NULL on error. */
io *iopenrb(const char *name);
/* Open a file as an io, read-write.  Returns NULL on error. */
io *iopenrw(const char *name);
/* Open a file as an io, write-only.  Returns NULL on error. */
io *iopenw(const char *name);
/* Open a file as a buffered io, write-only. Returns NULL on error. */
io *iopenwb(const char *name);
/* Open a file as an io, exclusively write-only.  Returns NULL on error. */
io *iopenwexcl(const char *name);
/* Open a file in append-only mode. Returns NULL on error. */
io *iopenwa(const char *name);
/* Create an io from an fd. Returns NULL on error. */
io *ifromfd(int fd);
/* Create an io from a socket. Returns NULL on error. */
io *ifromsock(int sock);
/* Create an io from another io, with read buffering. If close != 0,
   close from when the new io is closed. Returns NULL on error. */
io *irbuf(io *from, int bufsize, int close);
/* Create an io from another io, with write buffering. If close != 0,
   close from when the new io is closed. Returns NULL on error. */
io *iwbuf(io *from, int bufsize, int close);
/* Flush an io. Returns 0 on success, -1 on failure. */
int iflush(io *io);
/* Close an io, flushing it first. Returns 0 on success, -1 on failure. */
int iclose(io *io);
/* Read up to n bytes from an io. Returns the number of bytes read,
   0 on EOF, -1 on error. */
int iread(io *io, char *buf, int n);
/* Write up to n bytes to an io. Returns the number of bytes written,
   -1 on error. */
int iwrite(io *io, const char *buf, int n);
/* Write n bytes to an io. Returns 0 on success, -1 on error. */
int iwritea(io *io, const char *buf, int n);
/* Synchronise the file attached to an fd to disk. Returns 0 on success,
   -1 on error. */
int isync(io *io);

/* Format the optional arguments onto the io. Usage is something like:
   iformat(some_io, type, arg, type, arg, ..., 0);
   Type can be:	'b' for buf *
                'c' for char *
                'i' for int (decimal)
                'I' for unsigned int (decimal)
                'x' for int (hex)
                'X' for unsigned int (hex)
                'l' for long (decimal)
                'L' for unsigned long (decimal)
		'a' for an IP address in network byte order as an unsigned long
   Returns 0 on success, -1 on error. */
int iformat(io *io, ...);
/* Like iformat, but formats onto a buffer. */
void bformat(buffer *b, ...);

